unit RealCounters;

interface

uses
  Windows, Classes, SysUtils, Counters, adCpuUsage;

type
  TCPUCounter = class(TPercentCounter)
  private
    FCPUNumber : integer;
    FCPUTotal : integer;
  protected
    function GetTitle: string; override;
    function GetValue: extended; override;
  public
    constructor Create(CpuNumber, CpuTotal: integer);
  end;

  TMemLoadCounter = class(TPercentCounter)
  protected
    function GetTitle: string; override;
    function GetValue: extended; override;
  end;

  TVirtLoadCounter = class(TPercentCounter)
  protected
    function GetTitle: string; override;
    function GetValue: extended; override;
  end;

  TSwapLoadCounter = class(TPercentCounter)
  protected
    function GetTitle: string; override;
    function GetValue: extended; override;
  end;

procedure RefreshCounters;

implementation

{ =========================================================================== }

var
  MemState: TMEMORYSTATUS;

procedure RefreshCounters;
begin
  GlobalMemoryStatus(MemState);
  CollectCPUData;
end;

{ =========================================================================== }

function CountPercentage(Avail, Total: integer): extended;
var r1, r2: TLargeInteger;
begin
{$IFDEF VER90}
  r1.QuadPart := 0; r1.LowPart := Avail;
  r2.QuadPart := 0; r2.LowPart := Total;
  Result := (1 - (r1.QuadPart / r2.QuadPart)) * 100;
{$ELSE}
  r1 := Avail;
  r2 := Total;
  Result := (1 - (r1 / r2)) * 100;
{$ENDIF}
end;

{ =========================================================================== }
{ TCPUCounter }

constructor TCPUCounter.Create(CpuNumber, CpuTotal: integer);
begin
  inherited Create;
  FCpuNumber := CpuNumber;
  FCpuTotal := CpuTotal;
end;

function TCPUCounter.GetTitle: string;
begin
  if (FCpuTotal <= 1) then begin
    Result := ' CPU'
  end else
  if ((FCpuNumber + 1) = FCpuTotal) then begin
    Result := ' CPU ()';
  end else begin
    Result := Format(' CPU %d', [FCpuNumber + 1]);
  end;
end;

function TCPUCounter.GetValue: extended;
begin
  Result := GetCPUUsage(FCpuNumber) * 100;
  if (Result < 0) then Result := 0;
  if (Result > 100) then Result := 100;  
end;

{ =========================================================================== }
{ TMemLoadCounter }

function TMemLoadCounter.GetTitle: string;
begin
  Result := ' . ';
end;

function TMemLoadCounter.GetValue: extended;
begin
  Result := CountPercentage(MemState.dwAvailPhys, MemState.dwTotalPhys);
end;

{ =========================================================================== }
{ TMemLoadCounter }

function TVirtLoadCounter.GetTitle: string;
begin
  Result := ' . ';
end;

function TVirtLoadCounter.GetValue: extended;
begin
  Result := CountPercentage(MemState.dwAvailVirtual, MemState.dwTotalVirtual);
end;

{ =========================================================================== }
{ TSwapLoadCounter }

function TSwapLoadCounter.GetTitle: string;
begin
  Result := ' swap';
end;

function TSwapLoadCounter.GetValue: extended;
begin
  Result := CountPercentage(MemState.dwAvailPageFile, MemState.dwTotalPageFile);
end;

{ =========================================================================== }

procedure RegisterCounters;
var i, j: integer;
begin
  CollectCPUData;
  j := GetCPUCount;
  for i := 0 to j - 1 do
    RegisterCounter(TCPUCounter.Create(i, j));
  RegisterCounter(TMemLoadCounter.Create);
  RegisterCounter(TVirtLoadCounter.Create);
  RegisterCounter(TSwapLoadCounter.Create);
end;

{ =========================================================================== }

initialization
  RegisterCounters;

finalization

end.
